@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2006 Sam Lantinga
+    Copyright (C) 1997-2012 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,8 +47,8 @@ static void UpdateSpecialKeys(int special_keys_state);
 
 void AtariBios_InitOSKeymap(_THIS)
 {
-	int i, vectors_mask;
-	unsigned long dummy;
+	int vectors_mask;
+/*	unsigned long dummy;*/
 
 	SDL_memset(bios_currentkeyboard, 0, sizeof(bios_currentkeyboard));
 	SDL_memset(bios_previouskeyboard, 0, sizeof(bios_previouskeyboard));
@@ -71,6 +71,8 @@ void AtariBios_PumpEvents(_THIS)
 	int i;
 	SDL_keysym keysym;
 
+	SDL_AtariMint_BackgroundTasks();
+
 	/* Update pressed keys */
 	SDL_memset(bios_currentkeyboard, 0, ATARIBIOS_MAXKEYS);
 
